// Generated by CoffeeScript 1.10.0
(function() {
  var KicadGenerator, fs, mkdirp, sprintf;

  fs = require('fs');

  mkdirp = require('mkdirp');

  sprintf = require('sprintf-js').sprintf;

  KicadGenerator = (function() {
    function KicadGenerator(library) {
      this.library = library;
      this.f = "%." + this.library.pattern.decimals + "f";
    }

    KicadGenerator.prototype.generate = function(name) {
      var dir, element, fd, j, len, now, pattern, patternName, patterns, ref, results, timestamp;
      dir = './kicad';
      mkdirp.sync(dir + "/" + name + ".pretty");
      patterns = {};
      now = new Date;
      timestamp = (now.getDate()) + "/" + (now.getMonth() + 1) + "/" + (now.getYear() + 1900) + " " + (now.getHours()) + ":" + (now.getMinutes()) + ":" + (now.getSeconds());
      fd = fs.openSync(dir + "/" + name + ".lib", 'w');
      fs.writeSync(fd, "EESchema-LIBRARY Version 2.3 Date: " + timestamp + "\n");
      fs.writeSync(fd, '#encoding utf-8\n');
      ref = this.library.elements;
      for (j = 0, len = ref.length; j < len; j++) {
        element = ref[j];
        this._generateSymbol(fd, element);
        patterns[element.pattern.name] = element.pattern;
      }
      fs.writeSync(fd, '# End Library\n');
      fs.closeSync(fd);
      console.log("Generating KiCad library '" + name + ".lib': OK");
      results = [];
      for (patternName in patterns) {
        pattern = patterns[patternName];
        fd = fs.openSync(dir + "/" + name + ".pretty/" + patternName + ".kicad_mod", 'w');
        this._generatePattern(fd, pattern);
        console.log("Generating KiCad footprint '" + patternName + ".kicad_mod': OK");
        results.push(fs.closeSync(fd));
      }
      return results;
    };

    KicadGenerator.prototype._generatePattern = function(fd, pattern) {
      var j, len, patObj, ref, shape;
      fs.writeSync(fd, "(module " + pattern.name + " (layer F.Cu)\n");
      if (pattern.type === 'smd') {
        fs.writeSync(fd, "  (attr smd)\n");
      }
      ref = pattern.shapes;
      for (j = 0, len = ref.length; j < len; j++) {
        shape = ref[j];
        patObj = this._patternObj(shape);
        switch (patObj.kind) {
          case 'attribute':
            fs.writeSync(fd, sprintf("  (fp_text %s %s (at " + this.f + " " + this.f + ") (layer %s)\n", patObj.name, patObj.text, patObj.x, patObj.y, patObj.layer));
            fs.writeSync(fd, sprintf("    (effects (font (size " + this.f + " " + this.f + ") (thickness " + this.f + ")))\n", patObj.fontSize, patObj.fontSize, patObj.lineWidth));
            fs.writeSync(fd, "  )\n");
            break;
          case 'circle':
            fs.writeSync(fd, sprintf("  (fp_circle (center " + this.f + " " + this.f + ") (end " + this.f + " " + this.f + ") (layer %s) (width " + this.f + "))\n", patObj.x, patObj.y, patObj.x, patObj.y + patObj.radius, patObj.layer, patObj.lineWidth));
            break;
          case 'line':
            fs.writeSync(fd, sprintf("  (fp_line (start " + this.f + " " + this.f + ") (end " + this.f + " " + this.f + ") (layer %s) (width " + this.f + "))\n", patObj.x1, patObj.y1, patObj.x2, patObj.y2, patObj.layer, patObj.lineWidth));
            break;
          case 'pad':
            fs.writeSync(fd, sprintf("  (pad %s %s %s (at " + this.f + " " + this.f + ") (size " + this.f + " " + this.f + ") (layers %s))\n", patObj.name, patObj.type, patObj.shape, patObj.x, patObj.y, patObj.width, patObj.height, patObj.layer));
        }
      }
      return fs.writeSync(fd, ")\n");
    };

    KicadGenerator.prototype._generateSymbol = function(fd, element) {
      var attrObj, i, j, k, l, len, len1, len2, nameObj, pinNameSpace, ref, ref1, ref2, ref3, ref4, refObj, shape, showPinNames, showPinNumbers, symObj, symbol;
      ref = element.symbols;
      for (j = 0, len = ref.length; j < len; j++) {
        symbol = ref[j];
        symbol.invertVertical();
      }
      symbol = element.symbols[0];
      refObj = this._symbolObj(symbol.attributes['refDes']);
      nameObj = this._symbolObj(symbol.attributes['name']);
      fs.writeSync(fd, "#\n# " + element.name + "\n#\n");
      showPinNumbers = ((ref1 = element.schematic) != null ? ref1.showPinNumbers : void 0) ? 'Y' : 'N';
      showPinNames = ((ref2 = element.schematic) != null ? ref2.showPinNames : void 0) ? 'Y' : 'N';
      pinNameSpace = Math.round(this.library.symbol.space.pinName);
      fs.writeSync(fd, "DEF " + element.name + " " + element.refDes + " 0 " + pinNameSpace + " " + showPinNumbers + " " + showPinNames + " " + element.symbols.length + " L N\n");
      fs.writeSync(fd, "F0 \"" + element.refDes + "\" " + refObj.x + " " + refObj.y + " " + refObj.fontSize + " " + refObj.orientation + " V " + refObj.halign + " " + refObj.valign + "NN\n");
      fs.writeSync(fd, "F1 \"" + element.name + "\" " + nameObj.x + " " + nameObj.y + " " + nameObj.fontSize + " " + nameObj.orientation + " V " + nameObj.halign + " " + nameObj.valign + "NN\n");
      if (symbol.attributes['user'] != null) {
        attrObj = this._symbolObj(symbol.attributes['user']);
        fs.writeSync(fd, "F4 \"" + attrObj.text + "\" " + attrObj.x + " " + attrObj.y + " " + attrObj.fontSize + " " + attrObj.orientation + " V " + attrObj.halign + " " + attrObj.valign + "NN\n");
      }
      if (element.aliases != null) {
        fs.writeSync(fd, "ALIAS " + (element.aliases.join(' ')) + "\n");
      }
      fs.writeSync(fd, "$FPLIST\n");
      fs.writeSync(fd, "  " + element.pattern.name + "\n");
      fs.writeSync(fd, "$ENDFPLIST\n");
      fs.writeSync(fd, "DRAW\n");
      i = 1;
      ref3 = element.symbols;
      for (k = 0, len1 = ref3.length; k < len1; k++) {
        symbol = ref3[k];
        ref4 = symbol.shapes;
        for (l = 0, len2 = ref4.length; l < len2; l++) {
          shape = ref4[l];
          symObj = this._symbolObj(shape);
          switch (symObj.kind) {
            case 'pin':
              fs.writeSync(fd, "X " + symObj.name + " " + symObj.number + " " + symObj.x + " " + symObj.y + " " + symObj.length + " " + symObj.orientation + " " + symObj.fontSizeNum + " " + symObj.fontSizeName + " " + i + " 1 " + symObj.type + symObj.shape + "\n");
              break;
            case 'rectangle':
              fs.writeSync(fd, "S " + symObj.x + " " + symObj.y + " " + (symObj.x + symObj.width) + " " + (symObj.y + symObj.height) + " " + i + " 1 0 " + symObj.fillStyle + "\n");
          }
        }
        ++i;
      }
      fs.writeSync(fd, "ENDDRAW\n");
      return fs.writeSync(fd, "ENDDEF\n");
    };

    KicadGenerator.prototype._patternObj = function(shape) {
      var obj;
      obj = shape;
      if (obj.shape === 'rectangle') {
        obj.shape = 'rect';
      }
      obj.layer = (function() {
        switch (obj.layer) {
          case 'top':
            if (obj.kind === 'pad') {
              return 'F.Cu F.Paste F.Mask';
            } else {
              return 'F.Cu';
            }
            break;
          case 'topSilkscreen':
            return 'F.SilkS';
          case 'topAssembly':
            return 'F.Fab';
          case 'bottom':
            if (obj.kind === 'pad') {
              return 'B.Cu B.Paste B.Mask';
            } else {
              return 'B.Cu';
            }
            break;
          case 'bottomSilkscreen':
            return 'B.SilkS';
          case 'bottomAssembly':
            return 'B.Fab';
          default:
            return 'F.Cu';
        }
      })();
      if (obj.kind === 'attribute') {
        switch (obj.name) {
          case 'refDes':
            obj.name = 'reference';
            obj.text = 'REF**';
            if (obj.fontSize == null) {
              obj.fontSize = this.library.pattern.fontSize.refDes;
            }
            break;
          case 'value':
            if (obj.fontSize == null) {
              obj.fontSize = this.library.pattern.fontSize.value;
            }
            break;
          default:
            obj.name = 'user';
            if (obj.fontSize == null) {
              obj.fontSize = this.library.pattern.fontSize["default"];
            }
        }
      }
      return obj;
    };

    KicadGenerator.prototype._symbolObj = function(shape) {
      var obj;
      obj = shape;
      obj.x = Math.round(obj.x);
      obj.y = Math.round(obj.y);
      obj.length = Math.round(obj.length);
      obj.halign = (function() {
        switch (obj.halign) {
          case 'center':
            return 'C';
          case 'right':
            return 'R';
          default:
            return 'L';
        }
      })();
      obj.valign = (function() {
        switch (obj.valign) {
          case 'center':
            return 'C';
          case 'bottom':
            return 'B';
          default:
            return 'T';
        }
      })();
      obj.orientation = (function() {
        switch (obj.orientation) {
          case 'left':
            return 'L';
          case 'right':
            return 'R';
          case 'up':
            return 'U';
          case 'down':
            return 'D';
          case 'horizontal':
            return 'H';
          case 'vertical':
            return 'V';
          default:
            return 'H';
        }
      })();
      obj.fontSize = Math.round(this.library.symbol.fontSize[shape.name] != null ? this.library.symbol.fontSize[shape.name] : this.library.symbol.fontSize["default"]);
      obj.fontSizeNum = Math.round(this.library.symbol.fontSize.pinNumber);
      obj.fontSizeName = Math.round(this.library.symbol.fontSize.pinName);
      obj.type = 'U';
      if (obj.power || obj.ground) {
        obj.type = 'W';
        if (obj.output) {
          obj.type = 'w';
        }
      } else {
        if (obj.bidir || (obj["in"] && obj.out)) {
          obj.type = 'B';
        } else if (obj["in"]) {
          obj.type = 'I';
        } else if (obj.out) {
          obj.type = 'O';
        } else if (obj.nc) {
          obj.type = 'N';
        } else if (obj.z) {
          obj.type = 'T';
        } else {
          obj.type = 'U';
        }
      }
      obj.shape = '';
      if (obj.invisible) {
        obj.shape += 'N';
      }
      if (obj.inverted) {
        obj.shape += 'I';
      } else if (obj.nc) {
        obj.shape += 'NX';
      }
      if (obj.shape !== '') {
        obj.shape = ' ' + obj.shape;
      }
      switch (obj.fill) {
        case 'foreground':
          obj.fillStyle = 'f';
          break;
        case 'backgroung':
          obj.fillStyle = 'F';
          break;
        default:
          obj.fillStyle = 'N';
      }
      return obj;
    };

    return KicadGenerator;

  })();

  module.exports = KicadGenerator;

}).call(this);
