// Generated by CoffeeScript 1.10.0
(function() {
  var KicadGenerator, QedaElement, QedaLibrary, fs, mkdirp, path, request, yaml;

  fs = require('fs');

  mkdirp = require('mkdirp');

  path = require('path');

  request = require('sync-request');

  yaml = require('js-yaml');

  KicadGenerator = require('./kicad-generator');

  QedaElement = require('./qeda-element');

  QedaLibrary = (function() {
    function QedaLibrary(settings) {
      if (settings == null) {
        settings = {};
      }
      this.elementStyle = 'default';
      this.symbolStyle = 'default';
      this.patternStyle = 'default';
      this.output = 'kicad';
      this.symbol = {
        units: 'mil',
        gridSize: 50,
        fontSize: {
          "default": 1,
          refDes: 1,
          name: 1,
          pinNumber: 1,
          pinName: 1
        },
        lineWidth: {
          "default": 0.02
        },
        space: {
          pinName: 1
        }
      };
      this.pattern = {
        densityLevel: 'N',
        decimals: 3,
        tolerance: {
          "default": 0.1,
          fabrication: 0.05,
          placement: 0.05
        },
        clearance: {
          padToSilk: 0.2
        },
        lineWidth: {
          "default": 0.2,
          silkscreen: 0.2,
          assembly: 0.1
        },
        fontSize: {
          "default": 1,
          refDes: 1,
          value: 1
        }
      };
      this.mergeObjects(this, settings);
      this.elements = [];
    }

    QedaLibrary.prototype.add = function(element) {
      var aliases, base1, def, housing, housings, i, j, key, keys, len, len1, name, newElement, res, suffixes;
      def = this.load(element);
      if (def.abstract) {
        console.error("'" + element + "': Cannot add abstract component, use it only as base for others");
        process.exit(1);
      }
      res = [];
      housings = [];
      if (typeof def.housing === 'string') {
        keys = def.housing.replace(/\s+/g, '').split(',');
        for (i = 0, len = keys.length; i < len; i++) {
          key = keys[i];
          housing = def[key];
          if (housing.suffix == null) {
            housing.suffix = key;
          }
          housings.push(housing);
        }
      } else {
        if ((base1 = def.housing).suffix == null) {
          base1.suffix = '';
        }
        housings.push(def.housing);
      }
      name = def.name;
      for (j = 0, len1 = housings.length; j < len1; j++) {
        housing = housings[j];
        suffixes = housing.suffix.replace(/\s+/g, '').split(',');
        def.name = name + suffixes[0];
        def.housing = housing;
        if (suffixes.length > 1) {
          aliases = [];
          if (def.alias != null) {
            aliases.concat(def.alias.replace(/\s+/g, '').split(','));
          }
          aliases.concat(suffixes.map((function(_this) {
            return function(a) {
              return name + a;
            };
          })(this)));
          def.aliases = aliases;
        }
        newElement = new QedaElement(this, def);
        this.elements.push(newElement);
        res.push(newElement);
      }
      return res;
    };

    QedaLibrary.prototype.calculate = function() {
      var element, i, j, l, len, len1, len2, prop, ref, ref1, ref2, symbol;
      if (this._calculated == null) {
        this._calculated = false;
      }
      if (this._calculated) {
        return;
      }
      ref = this.elements;
      for (i = 0, len = ref.length; i < len; i++) {
        element = ref[i];
        element.render();
      }
      for (prop in this.symbol.fontSize) {
        this.symbol.fontSize[prop] *= this.symbol.gridSize;
      }
      for (prop in this.symbol.lineWidth) {
        this.symbol.lineWidth[prop] *= this.symbol.gridSize;
      }
      for (prop in this.symbol.space) {
        this.symbol.space[prop] *= this.symbol.gridSize;
      }
      ref1 = this.elements;
      for (j = 0, len1 = ref1.length; j < len1; j++) {
        element = ref1[j];
        ref2 = element.symbols;
        for (l = 0, len2 = ref2.length; l < len2; l++) {
          symbol = ref2[l];
          symbol.resize(this.symbol.gridSize);
        }
      }
      return this._calculated = true;
    };

    QedaLibrary.prototype.generate = function(name) {
      var generator;
      this.calculate();
      generator = null;
      switch (this.output) {
        case 'kicad':
          generator = new KicadGenerator(this);
      }
      if (generator) {
        return generator.generate(name);
      }
    };

    QedaLibrary.prototype.load = function(element) {
      var base, baseDef, baseElement, bases, def, elementYaml, error, error1, error2, i, len, localFile, r, res;
      elementYaml = element.toLowerCase() + '.yaml';
      localFile = './library/' + elementYaml;
      if (!fs.existsSync(localFile)) {
        try {
          r = request.defaults({
            proxy: 'http://proxy.croc.ru:8000'
          });
          res = request('GET', 'https://raw.githubusercontent.com/qeda/library/master/' + elementYaml, {
            timeout: 3000
          });
        } catch (error1) {
          error = error1;
          console.error("Loading '" + element + "': Error: " + error.message);
          process.exit(1);
        }
        if (res.statusCode === 200) {
          mkdirp.sync(path.dirname(localFile));
          fs.writeFileSync(localFile, res.body);
          console.log("Loading '" + element + "': OK");
        } else {
          console.error("Loading '" + element + "': Error: (" + res.statusCode + ")");
          process.exit(1);
        }
      }
      try {
        def = yaml.safeLoad(fs.readFileSync(localFile));
      } catch (error2) {
        error = error2;
        console.error("Loading '" + element + "': Error: " + error.message);
        process.exit(1);
      }
      if (def.base != null) {
        bases = def.base.replace(/\s+/g, '').split(',');
        delete def.base;
        for (i = 0, len = bases.length; i < len; i++) {
          base = bases[i];
          baseElement = base;
          if (path.dirname(baseElement) === '.') {
            baseElement = path.dirname(element) + '/' + baseElement;
          }
          baseDef = this.load(baseElement);
          if (baseDef.abstract) {
            delete baseDef.abstract;
          }
          this.mergeObjects(baseDef, def);
          def = baseDef;
        }
      }
      return def;
    };

    QedaLibrary.prototype.mergeObjects = function(dest, src) {
      var k, results, v;
      results = [];
      for (k in src) {
        v = src[k];
        if (typeof v === 'object' && (!Array.isArray(v)) && dest.hasOwnProperty(k)) {
          results.push(this.mergeObjects(dest[k], v));
        } else {
          results.push(dest[k] = v);
        }
      }
      return results;
    };

    QedaLibrary.prototype.setElementStyle = function(style) {
      style = style.toLowerCase();
      this.elementStyle = style;
      if (this.symbolStyle === 'default' || style === 'default') {
        this.symbolStyle = style;
      }
      if (this.patternStyle === 'default' || style === 'default') {
        return this.patternStyle = style;
      }
    };

    QedaLibrary.prototype.setPatternStyle = function(style) {
      return this.patternStyle = style.toLowerCase();
    };

    QedaLibrary.prototype.setSymbolStyle = function(style) {
      return this.symbolStyle = style.toLowerCase();
    };

    return QedaLibrary;

  })();

  module.exports = QedaLibrary;

}).call(this);
